searchNodes=[{"doc":"Core library for a Chip-8 interpreter. This library only handles the core logic to run Chip-8 programs and does not handle any I/O, which allows it to be easily embedded into a UI. Created by Joseph Weisbecker in the mid-1970s, Chip-8 is a programming language composed of hexadecimal instructions and are executed on the fly by a virtual machine. The language was created to make it easier to develop video games for the COSMAC VIP microcomputer. After becoming popular because of its simplicity, people started to implement interpreter versions for other platforms but, because the language did not have a formal specification, many of these versions some instruction had some deviations on their behavior compared to the original COSMAC VIP version. This project follows the Cowgod's Chip-8 Technical Reference v1.0 specification and partially replicates the specification on its own documentation, being able to act as standalone documentation if needed. Usage To run programs, you must initialize an interpreter instance through Chip8.initialize/1 : Chip8 . initialize ( program_data ) The returned interpreter instance holds all the required states for the program's execution at a given point in time. Since the instance is just a pure data structure, your application will be the one responsible to manage everything, from user interaction to managing cycles and their relation with timers. See all Chip8 functions for more information about each operation.","ref":"Chip8.html","title":"Chip8","type":"module"},{"doc":"Changes the cycle rate of an interpreter. iex&gt; Chip8 . change_cycle_rate ( interpreter , 6 ) % Chip8.Interpreter { }","ref":"Chip8.html#change_cycle_rate/2","title":"Chip8.change_cycle_rate/2","type":"function"},{"doc":"Executes a single instruction cycle. A cycle is composed of 3 steps: fetch the instruction from memory, decode the instruction bytes and executes it by changing the interpreter's state. See the cycle section at Chip8.Interpreter for more information. iex&gt; Chip8 . cycle ( interpreter ) % Chip8.Interpreter { }","ref":"Chip8.html#cycle/1","title":"Chip8.cycle/1","type":"function"},{"doc":"Shortcut to Chip8.initialize/2 . iex&gt; program = File . read! ( path ) iex&gt; Chip8 . initialize ( program ) % Chip8.Interpreter { }","ref":"Chip8.html#initialize/1","title":"Chip8.initialize/1","type":"function"},{"doc":"Creates an interpreter instance with the given program loaded into its memory. Options cycle_rate - amount of instructions that are going to be executed each cycle. Defaults to 10 when not specified. iex&gt; program = File . read! ( path ) iex&gt; Chip8 . initialize ( program , cycle_rate : 10 ) % Chip8.Interpreter { }","ref":"Chip8.html#initialize/2","title":"Chip8.initialize/2","type":"function"},{"doc":"Returns a pixelmap with the current content of the display. This pixelmap is a list matrix with each pixel &quot;state&quot; of the whole display. See Chip8.Interpreter.Display for more information. iex&gt; Chip8 . pixelmap ( interpreter ) [ [ 0 , 0 , 1 , ... ] , [ 0 , 0 , ... ] , [ 1 , ... ] , [ ... ] , ... ]","ref":"Chip8.html#pixelmap/1","title":"Chip8.pixelmap/1","type":"function"},{"doc":"Computes a key press event on the hexadecimal keyboard. See Chip8.Interpreter.Keyboard for more information. iex&gt; Chip8 . press_key ( interpreter , 0xA ) % Chip8.Interpreter { }","ref":"Chip8.html#press_key/2","title":"Chip8.press_key/2","type":"function"},{"doc":"Computes a key release event on the hexadecimal keyboard. See Chip8.Interpreter.Keyboard for more information. iex&gt; Chip8 . release_key ( interpreter , 0xA ) % Chip8.Interpreter { }","ref":"Chip8.html#release_key/2","title":"Chip8.release_key/2","type":"function"},{"doc":"Performs the tick logic for all timers. This function does not set timers to decrement at the required rate, only performing the tick itself. See Chip8.Interpreter.Timer for more information on timers. iex&gt; Chip8 . tick_timers ( interpreter ) % Chip8.Interpreter { }","ref":"Chip8.html#tick_timers/1","title":"Chip8.tick_timers/1","type":"function"},{"doc":"An interpreter for the Chip-8 language. The interpreter uses the following components to run a program: Display A monochrome display used to render graphics. See Chip8.Interpreter.Display for more information. Memory A memory space of 4Kb used to store programs and data. See Chip8.Interpreter.Memory for more information. Delay Timer A timer used to coordinate events within a program, usually referred to as dt . See Chip8.Interpreter.Timer for more information. Sound Timer A timer used to emit a monotone sound while it's active, usually referred to as st . See Chip8.Interpreter.Timer for more information. Stack A stack to store and retrieve memory addresses when calling and returning from subroutines. Program Counter A 16-bit integer that points to the memory address of the current instruction, and is usually referred to as pc . I A 16-bit register used to point to memory locations, usually to render sprites. Variable Registers 16 8-bit registers to store and retrieve variable data. See Chip8.Interpreter.VRegisters for more information. Note that some of the components above are only accessible to programs through specific instructions (e.g. display and memory components) and some are not accessible at all (i.e. program counter). Cycle At each cycle, the interpreter performs the same sequence of steps until the the program reaches its end: Fetch 2 bytes of memory from the address that pc is pointing at; Decode the fetched bytes into an opcode with its operands; Execute the opcode by updating the interpreter state based on the opcode specification. All programs are executed through this simple routine, so to prevent the interpreter to stop running they create an infinite loop (either a direct or indirect one) and keep executing until the user quits the application.","ref":"Chip8.Interpreter.html","title":"Chip8.Interpreter","type":"module"},{"doc":"","ref":"Chip8.Interpreter.html#change_cycle_rate/2","title":"Chip8.Interpreter.change_cycle_rate/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#cycle/1","title":"Chip8.Interpreter.cycle/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#get_font_character_address/1","title":"Chip8.Interpreter.get_font_character_address/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#initialize/2","title":"Chip8.Interpreter.initialize/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#load_font/2","title":"Chip8.Interpreter.load_font/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#load_program/2","title":"Chip8.Interpreter.load_program/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#new/1","title":"Chip8.Interpreter.new/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#pixelmap/1","title":"Chip8.Interpreter.pixelmap/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#press_key/2","title":"Chip8.Interpreter.press_key/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#release_key/2","title":"Chip8.Interpreter.release_key/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#tick_timers/1","title":"Chip8.Interpreter.tick_timers/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#to_next_instruction/1","title":"Chip8.Interpreter.to_next_instruction/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#to_previous_instruction/1","title":"Chip8.Interpreter.to_previous_instruction/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.html#t:t/0","title":"Chip8.Interpreter.t/0","type":"type"},{"doc":"A simple Last In First Out queue.","ref":"Chip8.Stack.html","title":"Chip8.Stack","type":"module"},{"doc":"","ref":"Chip8.Stack.html#new/0","title":"Chip8.Stack.new/0","type":"function"},{"doc":"","ref":"Chip8.Stack.html#pop/1","title":"Chip8.Stack.pop/1","type":"function"},{"doc":"","ref":"Chip8.Stack.html#push/2","title":"Chip8.Stack.push/2","type":"function"},{"doc":"","ref":"Chip8.Stack.html#t:t/0","title":"Chip8.Stack.t/0","type":"type"},{"doc":"A monochrome display device. The monochrome display has a resolution of 64x32 pixels (64 pixels wide and 32 pixels high), programs can render graphics on it through the use of sprites (see Chip8.Interpreter.Display.Sprite ) and coordinates. The origin point of the display is at the top-left corner, with the coordinates (0, 0) . Pixelmap In most cases, pixelmaps contain the color information of each pixel in the screen but, since the COSMAC VIP wasn't able to produce colors and there is no way to specify one with the DRW instruction, the only information returned is the pixel &quot;state&quot;, which can be, 1 when the pixel is &quot;on&quot; and 0 when is &quot;off&quot;.","ref":"Chip8.Interpreter.Display.html","title":"Chip8.Interpreter.Display","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Display.html#clear/1","title":"Chip8.Interpreter.Display.clear/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Display.html#create_sprite/1","title":"Chip8.Interpreter.Display.create_sprite/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Display.html#draw/3","title":"Chip8.Interpreter.Display.draw/3","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Display.html#get_coordinates/3","title":"Chip8.Interpreter.Display.get_coordinates/3","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Display.html#has_collision?/2","title":"Chip8.Interpreter.Display.has_collision?/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Display.html#new/2","title":"Chip8.Interpreter.Display.new/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Display.html#pixelmap/1","title":"Chip8.Interpreter.Display.pixelmap/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Display.html#t:coordinates/0","title":"Chip8.Interpreter.Display.coordinates/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Display.html#t:dimension/0","title":"Chip8.Interpreter.Display.dimension/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Display.html#t:pixel/0","title":"Chip8.Interpreter.Display.pixel/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Display.html#t:pixelmap/0","title":"Chip8.Interpreter.Display.pixelmap/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Display.html#t:t/0","title":"Chip8.Interpreter.Display.t/0","type":"type"},{"doc":"Group of bytes representing the binary structure of an image. Sprites may be up to 15 bytes , making it possible to have a maximum dimension of 8x15 pixels (8 pixels wide and 15 pixels high). When a sprite heigth is an odd number, the sprite should be padded in order to become an even number to not interfere with the instructions location pattern, see Chip8.Interpreter.Instruction for more information. Sprites are also used to represent the hexadecimal characters of the built-in font, in this case, the sprites will be 5 pixels wide, see Chip8.Interpreter.Font for more information.","ref":"Chip8.Interpreter.Display.Sprite.html","title":"Chip8.Interpreter.Display.Sprite","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Display.Sprite.html#new/1","title":"Chip8.Interpreter.Display.Sprite.new/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Display.Sprite.html#to_bitmap/1","title":"Chip8.Interpreter.Display.Sprite.to_bitmap/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Display.Sprite.html#t:bit/0","title":"Chip8.Interpreter.Display.Sprite.bit/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Display.Sprite.html#t:bitmap/0","title":"Chip8.Interpreter.Display.Sprite.bitmap/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Display.Sprite.html#t:t/0","title":"Chip8.Interpreter.Display.Sprite.t/0","type":"type"},{"doc":"Interpreter's built-in font sprites.","ref":"Chip8.Interpreter.Font.html","title":"Chip8.Interpreter.Font","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Font.html#character_byte_size/0","title":"Chip8.Interpreter.Font.character_byte_size/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Font.html#data/0","title":"Chip8.Interpreter.Font.data/0","type":"function"},{"doc":"Module to interact with interpreter's opcodes . The interpreter's instruction set includes 36 instructions, that allow programs to perform operations of arithmetic, control flow, data manipulation, graphics, and logical. All instructions are 2 bytes long and are stored in a big-endian format. In memory, the interpreter assumes that all instructions should start at an even address, and needs to be taken into account when adding sprite data, see Chip8.Interpreter.Display.Sprite for more information. Instruction Definition Instructions are grouped in modules that represent their operation and share the same mnemonic, making different opcodes as simple variants. Inside each module, there is a table describing all variants supported by the module, each variant specifies its opcode and the effects applied into the interpreter once it is invoked. Since each variant supports a different type of value as operands (i.e. arguments), the notation below is used to compactly describe the use of each part inside the opcode : nnn or address - A 12-bit integer value, representing an memory address n or nibble - A 4-bit integer value, representing a literal number kk or byte - An 8-bit integer value, representing a literal number x or Vx - A 4-bit integer value, representing one of the 16 variable registers y or Vy - A 4-bit integer value, representing one of the 16 variable registers Decode # When decoding an instruction, the given 2 bytes list is transformed into a Chip8.Interpreter.Instruction.t/0 , containing the module that supports the given opcode as one of its variants and the arguments, that was also parsed into its equivalent struct i.e. an nnn address is transformed into a Chip8.Interpreter.Instruction.Argument.Address.t/0 . In case an unknown segment of bytes is given, an error is returned.","ref":"Chip8.Interpreter.Instruction.html","title":"Chip8.Interpreter.Instruction","type":"behaviour"},{"doc":"","ref":"Chip8.Interpreter.Instruction.html#byte_size/0","title":"Chip8.Interpreter.Instruction.byte_size/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.html#decode/1","title":"Chip8.Interpreter.Instruction.decode/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.html#execute/2","title":"Chip8.Interpreter.Instruction.execute/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.html#c:execute/2","title":"Chip8.Interpreter.Instruction.execute/2","type":"callback"},{"doc":"","ref":"Chip8.Interpreter.Instruction.html#c:new/1","title":"Chip8.Interpreter.Instruction.new/1","type":"callback"},{"doc":"","ref":"Chip8.Interpreter.Instruction.html#new/2","title":"Chip8.Interpreter.Instruction.new/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.html#t:arguments/0","title":"Chip8.Interpreter.Instruction.arguments/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Instruction.html#t:t/0","title":"Chip8.Interpreter.Instruction.t/0","type":"type"},{"doc":"A 12-bit integer value representing a memory address location.","ref":"Chip8.Interpreter.Instruction.Argument.Address.html","title":"Chip8.Interpreter.Instruction.Argument.Address","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Address.html#new/1","title":"Chip8.Interpreter.Instruction.Argument.Address.new/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Address.html#new/3","title":"Chip8.Interpreter.Instruction.Argument.Address.new/3","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Address.html#t:t/0","title":"Chip8.Interpreter.Instruction.Argument.Address.t/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Address.html#t:value_range/0","title":"Chip8.Interpreter.Instruction.Argument.Address.value_range/0","type":"type"},{"doc":"An 8-bit integer literal value.","ref":"Chip8.Interpreter.Instruction.Argument.Byte.html","title":"Chip8.Interpreter.Instruction.Argument.Byte","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Byte.html#new/1","title":"Chip8.Interpreter.Instruction.Argument.Byte.new/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Byte.html#new/2","title":"Chip8.Interpreter.Instruction.Argument.Byte.new/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Byte.html#t:t/0","title":"Chip8.Interpreter.Instruction.Argument.Byte.t/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Byte.html#t:value_range/0","title":"Chip8.Interpreter.Instruction.Argument.Byte.value_range/0","type":"type"},{"doc":"A 4-bit integer literal value.","ref":"Chip8.Interpreter.Instruction.Argument.Nibble.html","title":"Chip8.Interpreter.Instruction.Argument.Nibble","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Nibble.html#new/1","title":"Chip8.Interpreter.Instruction.Argument.Nibble.new/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Nibble.html#t:t/0","title":"Chip8.Interpreter.Instruction.Argument.Nibble.t/0","type":"type"},{"doc":"A 4-bit integer value representing a data register or a label for one of the pseudo-registers . Pseudo-registers Name Description bcd binary-coded decimal, it represents a computation rather than an actual value. dt delay timer, see Chip8.Interpreter for more information. font represents the built-in font. i register I of the interpreter, see Chip8.Interpreter for more information. keyboard reads the keyboard and reports key events, see Chip8.Interpreter.Keyboard for more information. memory access to the memory space, see Chip8.Interpreter for more information. st sound timer, see Chip8.Interpreter for more information.","ref":"Chip8.Interpreter.Instruction.Argument.Register.html","title":"Chip8.Interpreter.Instruction.Argument.Register","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Register.html#bcd/0","title":"Chip8.Interpreter.Instruction.Argument.Register.bcd/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Register.html#dt/0","title":"Chip8.Interpreter.Instruction.Argument.Register.dt/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Register.html#font/0","title":"Chip8.Interpreter.Instruction.Argument.Register.font/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Register.html#i/0","title":"Chip8.Interpreter.Instruction.Argument.Register.i/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Register.html#keyboard/0","title":"Chip8.Interpreter.Instruction.Argument.Register.keyboard/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Register.html#memory/0","title":"Chip8.Interpreter.Instruction.Argument.Register.memory/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Register.html#st/0","title":"Chip8.Interpreter.Instruction.Argument.Register.st/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Register.html#v/1","title":"Chip8.Interpreter.Instruction.Argument.Register.v/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Instruction.Argument.Register.html#t:t/0","title":"Chip8.Interpreter.Instruction.Argument.Register.t/0","type":"type"},{"doc":"A keyboard input device. Users can interact with programs through a hexadecimal keypad that was present in the original Chip-8 computer. Keys are represented by hexadecimal integers. When a user presses the F key, the key needs to be registered as 15 instead. For this purpose, you can either use the hexadecimal notation (e.g. 0xF returns 15 ) for a hard-coded mapping or String.to_integer/2 (e.g. String.to_integer(&quot;F&quot;, 16) returns 15 ) for a more dynamic approach.","ref":"Chip8.Interpreter.Keyboard.html","title":"Chip8.Interpreter.Keyboard","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Keyboard.html#is_key/1","title":"Chip8.Interpreter.Keyboard.is_key/1","type":"macro"},{"doc":"","ref":"Chip8.Interpreter.Keyboard.html#is_pressed?/2","title":"Chip8.Interpreter.Keyboard.is_pressed?/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Keyboard.html#keys/0","title":"Chip8.Interpreter.Keyboard.keys/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Keyboard.html#new/0","title":"Chip8.Interpreter.Keyboard.new/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Keyboard.html#press_key/2","title":"Chip8.Interpreter.Keyboard.press_key/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Keyboard.html#release_key/2","title":"Chip8.Interpreter.Keyboard.release_key/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Keyboard.html#t:key/0","title":"Chip8.Interpreter.Keyboard.key/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Keyboard.html#t:key_state/0","title":"Chip8.Interpreter.Keyboard.key_state/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Keyboard.html#t:t/0","title":"Chip8.Interpreter.Keyboard.t/0","type":"type"},{"doc":"Memory space available for programs to store and manipulate data. The Chip-8 interpreter has a total of 4Kb (4096 bytes) of memory space, that is accessible from address 0x000 ( 0 ) to 0xFFF ( 4095 ). Originally, the interpreter implementation shared the same memory space of programs and occupied the sector between 0x000 ( 0 ) and 0x1FF ( 511 ), for this reason, programs should not use the interpreter space so they usually start at location 0x200 ( 512 ).","ref":"Chip8.Interpreter.Memory.html","title":"Chip8.Interpreter.Memory","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Memory.html#new/1","title":"Chip8.Interpreter.Memory.new/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Memory.html#read/3","title":"Chip8.Interpreter.Memory.read/3","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Memory.html#write/3","title":"Chip8.Interpreter.Memory.write/3","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Memory.html#t:data/0","title":"Chip8.Interpreter.Memory.data/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.Memory.html#t:t/0","title":"Chip8.Interpreter.Memory.t/0","type":"type"},{"doc":"A timer is an 8-bit integer that decrements at a constant rate. Timers can have two different states, active and inactive . A timer is active whenever its value is above 0, while in this state, the value decrements whenever a ticks occurs until it reaches 0, when it becomes inactive and stops decrementing. On each tick, the value decrements on steps of 1, and they should run at a rate of 60Hz . Such rate needs to configured externally, it is common that emulators tie up this with the display's refresh rate.","ref":"Chip8.Interpreter.Timer.html","title":"Chip8.Interpreter.Timer","type":"module"},{"doc":"","ref":"Chip8.Interpreter.Timer.html#new/0","title":"Chip8.Interpreter.Timer.new/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Timer.html#new/1","title":"Chip8.Interpreter.Timer.new/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Timer.html#tick/1","title":"Chip8.Interpreter.Timer.tick/1","type":"function"},{"doc":"","ref":"Chip8.Interpreter.Timer.html#t:t/0","title":"Chip8.Interpreter.Timer.t/0","type":"type"},{"doc":"General purpose variable registers. Registers are numbered from 0 to F and have a size of 8 bits. They are usually refered as V registers through the notation Vx , where x is a hexadecimal number. Some instructions use VF as a flag register, setting it to either 0 or 1 depending of the operation, for this reason most programs tend to avoid relying on this register to store data.","ref":"Chip8.Interpreter.VRegisters.html","title":"Chip8.Interpreter.VRegisters","type":"module"},{"doc":"","ref":"Chip8.Interpreter.VRegisters.html#get/2","title":"Chip8.Interpreter.VRegisters.get/2","type":"function"},{"doc":"","ref":"Chip8.Interpreter.VRegisters.html#new/0","title":"Chip8.Interpreter.VRegisters.new/0","type":"function"},{"doc":"","ref":"Chip8.Interpreter.VRegisters.html#set/3","title":"Chip8.Interpreter.VRegisters.set/3","type":"function"},{"doc":"","ref":"Chip8.Interpreter.VRegisters.html#t:register/0","title":"Chip8.Interpreter.VRegisters.register/0","type":"type"},{"doc":"","ref":"Chip8.Interpreter.VRegisters.html#t:t/0","title":"Chip8.Interpreter.VRegisters.t/0","type":"type"},{"doc":"Adds the first operand with the second operand. This operation always returns an 8-bit number, that is stored in the first operand. When an _overflow happens, the result is always wrapped to fit into the specified size. In some variants a carry bit is also returned, when an overflow happens the carry bit will be 1 , and 0 otherwise. Variants Opcode Mnemonic Description 7xkk ADD Vx, byte Set Vx = Vx + byte . 8xy4 ADD Vx, Vy Set Vx = Vx + Vy and VF = carry . Fx1E ADD I, Vx Set I = I + Vx .","ref":"Chip8.Interpreter.Instruction.ADD.html","title":"Chip8.Interpreter.Instruction.ADD","type":"module"},{"doc":"Calculates the bitwise AND between two operands. The result of this operation is saved into the first operand. A bitwise AND compares the corresponding bits of the values, when both of the bits are 1 it results in a 1, and 0 otherwise. Variants Opcode Mnemonic Description 8xy2 ADD Vx, Vy Set Vx = Vy AND Vy .","ref":"Chip8.Interpreter.Instruction.AND.html","title":"Chip8.Interpreter.Instruction.AND","type":"module"},{"doc":"Call a subroutine. Variants Opcode Mnemonic Description 2nnn CALL address Push the current pc to the stack and then set pc = address .","ref":"Chip8.Interpreter.Instruction.CALL.html","title":"Chip8.Interpreter.Instruction.CALL","type":"module"},{"doc":"Clear the display's contents. Variants Opcode Mnemonic Description 00E0 CLS Clear the display's contents.","ref":"Chip8.Interpreter.Instruction.CLS.html","title":"Chip8.Interpreter.Instruction.CLS","type":"module"},{"doc":"Render sprites into the display. The sprite bytes are XORed against the corresponding bytes in the display, when a sprite is positioned in a way that part of it is located outside of the display coordinates, only the part considered inside of the display is rendered. A collision flag is also returned and its value depends on the previous content of the display, if one or more pixels were erased during the current render call the collision flag is set to 1, and 0 otherwise. Variants Opcode Mnemonic Description Dxyn DRW Vx, Vy, nibble Display a n-byte sprite, stored on memory starting at location I , in the coordinates (Vx, Vy) and set VF = collision .","ref":"Chip8.Interpreter.Instruction.DRW.html","title":"Chip8.Interpreter.Instruction.DRW","type":"module"},{"doc":"Jump to a memory address. Variants Opcode Mnemonic Description 1nnn JP address Set pc = address . Bnnn JP V0, address Set pc = V0 + address .","ref":"Chip8.Interpreter.Instruction.JP.html","title":"Chip8.Interpreter.Instruction.JP","type":"module"},{"doc":"Loads a value into memory, registers, or timers. Most of the addressing variations simply loads the value of the second operand into the first operand but some posses a special behavior and allow to interact with components that are not accessible through registers or that perform calculations before saving the data, see &quot;Pseudo-registers&quot; section at Chip8.Interpreter.Instruction.Argument.Register for more information. Variants Opcode Mnemonic Description 6xkk LD Vx, byte Set Vx = byte . 8xy0 LD Vx, Vy Set Vx = Vy . Annn LD I, address Set I = address . Fx07 LD Vx, DT Set Vx = DT . Fx0A LD Vx, KEY Wait for a keypress and set Vx = KEY . Fx15 LD DT, Vx Set DT = Vx . Fx18 LD ST, Vx Set ST = Vx . Fx29 LD FONT, Vx Set I = font character addres at Vx . Fx33 LD BCD, Vx Write a binary-coded decimal representation of Vx in memory location I , I + 1 , and I + 2 . Fx55 LD I, Vx Write the value of registers V0 through Vx in memory, starting at location I . Fx65 LD Vx, I Write to registers V0 through Vx from memory, starting at location I .","ref":"Chip8.Interpreter.Instruction.LD.html","title":"Chip8.Interpreter.Instruction.LD","type":"module"},{"doc":"Calculates the bitwise OR between two operands. The result of this operation is stored in the first operand. A bitwise OR compares the corresponding bits of the values, when either of the bits is 1 it results in a 1, and 0 otherwise. Variants Opcode Mnemonic Description 8xy1 OR Vx, Vy Set Vx = Vx OR Vy .","ref":"Chip8.Interpreter.Instruction.OR.html","title":"Chip8.Interpreter.Instruction.OR","type":"module"},{"doc":"Return from a subroutine. In case there is not a subroutine to return from, nothing is done. Variants Opcode Mnemonic Description 00EE RET Set pc = address at the top of the stack .","ref":"Chip8.Interpreter.Instruction.RET.html","title":"Chip8.Interpreter.Instruction.RET","type":"module"},{"doc":"Generates a random 8-bit integer. The interpreter generates a random 8-bit integer and calculates the bitwise AND with the given byte. Variants Opcode Mnemonic Description Cxkk RND Vx, byte Set Vx = random byte AND byte .","ref":"Chip8.Interpreter.Instruction.RND.html","title":"Chip8.Interpreter.Instruction.RND","type":"module"},{"doc":"Skip the next instruction when values are equal. Variants Opcode Mnemonic Description 3xkk SE Vx, byte Skip next instruction if Vx == byte . 5xy0 SE Vx, Vy Skip next instruction if Vx == Vy .","ref":"Chip8.Interpreter.Instruction.SE.html","title":"Chip8.Interpreter.Instruction.SE","type":"module"},{"doc":"Calculates the result of an arithmetic left bitshift . The result of this operation is stored in the first operand. Variants Opcode Mnemonic Description 8xyE SHL Vx {, Vy} Set Vx = Vx SHL 1 and VF = Vx most-significant bit .","ref":"Chip8.Interpreter.Instruction.SHL.html","title":"Chip8.Interpreter.Instruction.SHL","type":"module"},{"doc":"Calculates the result of an arithmetic right bitshift . The result of this operation is saved in the first operand. Variants Opcode Mnemonic Description 8xy6 SHR Vx {, Vy} Set Vx = Vx SHR 1 and VF = Vx least-significant bit .","ref":"Chip8.Interpreter.Instruction.SHR.html","title":"Chip8.Interpreter.Instruction.SHR","type":"module"},{"doc":"Skip the next instruction when a specific key is not pressed. Variants Opcode Mnemonic Description ExA1 SKNP Vx Skip the next instruction if the key with the value of Vx is not pressed.","ref":"Chip8.Interpreter.Instruction.SKNP.html","title":"Chip8.Interpreter.Instruction.SKNP","type":"module"},{"doc":"Skip the next instruction when a specific key is pressed. Variants Opcode Mnemonic Description Ex9E SKP Vx Skip the next instruction if the key with the value of Vx is pressed.","ref":"Chip8.Interpreter.Instruction.SKP.html","title":"Chip8.Interpreter.Instruction.SKP","type":"module"},{"doc":"Skip the next instruction when values are not equal. Variants Opcode Mnemonic Description 4xkk SNE Vx, byte Skip the next instruction if Vx != byte . 9xy0 SNE Vx, Vy Skip the next instruction if Vx != Vy .","ref":"Chip8.Interpreter.Instruction.SNE.html","title":"Chip8.Interpreter.Instruction.SNE","type":"module"},{"doc":"Subtracts the first operand with the second operand. This operation always returns an 8-bit number, that is stored in the first operand. When an underflow happens, the result is always wrapped to fit into the specified size. In some variants a borrow bit is also returned, when an underflow happens the borrow bit will be 1 , and 0 otherwise. Variants Opcode Mnemonic Description 8xy5 SUB Vx, Vy Set Vx = Vx - Vy and VF = NOT borrow .","ref":"Chip8.Interpreter.Instruction.SUB.html","title":"Chip8.Interpreter.Instruction.SUB","type":"module"},{"doc":"Subtracts the second operand with the first operand. This operation always returns an 8-bit number, that is stored in the first operand. When an underflow happens, the result is always wrapped to fit into the specified size. In some variants a borrow bit is also returned, when an underflow happens the borrow bit will be 1 , and 0 otherwise. Variants Opcode Mnemonic Description 8xy7 SUBN Vx, Vy Set Vx = Vy - Vx and VF = NOT borrow .","ref":"Chip8.Interpreter.Instruction.SUBN.html","title":"Chip8.Interpreter.Instruction.SUBN","type":"module"},{"doc":"Call a native subroutine. It is not possible to call the native subroutines that were available in the computers where Chip-8 was introduced so this instruction is essentially a no-op. Variants Opcode Mnemonic Description 0nnn SYS address No-op.","ref":"Chip8.Interpreter.Instruction.SYS.html","title":"Chip8.Interpreter.Instruction.SYS","type":"module"},{"doc":"Calculates the bitwise XOR between two operands. The result of this operation is saved in the first operand. An exclusive OR compares the corresponding bits of the values, when neither of the bits are the same it results in a 1, and 0 otherwise. Variants Opcode Mnemonic Description 8xy3 XOR Vx, Vy Set Vx = Vx XOR Vy .","ref":"Chip8.Interpreter.Instruction.XOR.html","title":"Chip8.Interpreter.Instruction.XOR","type":"module"},{"doc":"Core library for a Chip-8 interpreter. This library only handles the core logic to run Chip-8 programs and does not handle any I/O, which allows it to be easily embedded into a UI. Created by Joseph Weisbecker in the mid-1970s, Chip-8 is a programming language composed of hexadecimal instructions and are executed on the fly by a virtual machine. The language was created to make it easier to develop video games for the COSMAC VIP microcomputer. After becoming popular because of its simplicity, people started to implement interpreter versions for other platforms but, because the language did not have a formal specification, many of these versions some instruction had some deviations on their behavior compared to the original COSMAC VIP version. This project follows the Cowgod's Chip-8 Technical Reference v1.0 specification and partially replicates the specification on its own documentation, being able to act as standalone documentation if needed.","ref":"readme.html","title":"Chip-8","type":"extras"},{"doc":"To run programs, you must initialize an interpreter instance through Chip8.initialize/1 : Chip8 . initialize ( program_data ) The returned interpreter instance holds all the required states for the program's execution at a given point in time. Since the instance is just a pure data structure, your application will be the one responsible to manage everything, from user interaction to managing cycles and their relation with timers. See all Chip8 functions for more information about each operation.","ref":"readme.html#usage","title":"Chip-8 - Usage","type":"extras"},{"doc":"The package can be installed by adding chip8 to your list of dependencies in mix.exs : def deps do [ { :chip8 , &quot;~&gt; 1.0&quot; } ] end The docs can be found at https://hexdocs.pm/chip8 .","ref":"readme.html#installation","title":"Chip-8 - Installation","type":"extras"}]